"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SnapshotV1 = void 0;
const client_utils_1 = require("@fluid-internal/client-utils");
const internal_1 = require("@fluidframework/core-utils/internal");
const internal_2 = require("@fluidframework/runtime-utils/internal");
const internal_3 = require("@fluidframework/telemetry-utils/internal");
const constants_js_1 = require("./constants.js");
const mergeTreeNodeWalk_js_1 = require("./mergeTreeNodeWalk.js");
const properties_js_1 = require("./properties.js");
const segmentInfos_js_1 = require("./segmentInfos.js");
const snapshotChunks_js_1 = require("./snapshotChunks.js");
const snapshotlegacy_js_1 = require("./snapshotlegacy.js");
const opstampUtils = __importStar(require("./stamps.js"));
class SnapshotV1 {
    constructor(mergeTree, logger, getLongClientId, filename, onCompletion) {
        this.mergeTree = mergeTree;
        this.getLongClientId = getLongClientId;
        this.filename = filename;
        this.onCompletion = onCompletion;
        this.logger = (0, internal_3.createChildLogger)({ logger, namespace: "Snapshot" });
        this.chunkSize = mergeTree?.options?.mergeTreeSnapshotChunkSize ?? SnapshotV1.chunkSize;
        const { currentSeq, minSeq } = mergeTree.collabWindow;
        this.header = {
            minSequenceNumber: minSeq,
            sequenceNumber: currentSeq,
            orderedChunkMetadata: [],
            totalLength: 0,
            totalSegmentCount: 0,
        };
        this.segments = [];
        this.segmentLengths = [];
        this.attributionCollections = [];
    }
    getSeqLengthSegs(allSegments, allLengths, attributionCollections, approxSequenceLength, startIndex = 0) {
        const segments = [];
        const collections = [];
        let length = 0;
        let segmentCount = 0;
        let hasAttribution = false;
        while (length < approxSequenceLength && startIndex + segmentCount < allSegments.length) {
            const pseg = allSegments[startIndex + segmentCount];
            segments.push(pseg);
            length += allLengths[startIndex + segmentCount];
            if (attributionCollections[startIndex + segmentCount]) {
                hasAttribution = true;
                collections.push({
                    attribution: attributionCollections[startIndex + segmentCount],
                    cachedLength: allLengths[startIndex + segmentCount],
                });
            }
            segmentCount++;
        }
        const attributionSerializer = this.mergeTree.attributionPolicy?.serializer;
        (0, internal_1.assert)(!hasAttribution || attributionSerializer !== undefined, 0x55a /* attribution serializer must be provided when there are segments with attribution. */);
        return {
            version: "1",
            segmentCount,
            length,
            segments,
            startIndex,
            headerMetadata: undefined,
            attribution: hasAttribution
                ? attributionSerializer?.serializeAttributionCollections(collections)
                : undefined,
        };
    }
    /**
     * Emits the snapshot to an ISummarizeResult. If provided the optional IFluidSerializer will be used when
     * serializing the summary data rather than JSON.stringify.
     */
    emit(serializer, bind) {
        const chunks = [];
        this.header.totalSegmentCount = 0;
        this.header.totalLength = 0;
        do {
            const chunk = this.getSeqLengthSegs(this.segments, this.segmentLengths, this.attributionCollections, this.chunkSize, this.header.totalSegmentCount);
            chunks.push(chunk);
            this.header.totalSegmentCount += chunk.segmentCount;
            this.header.totalLength += chunk.length;
        } while (this.header.totalSegmentCount < this.segments.length);
        // The do while loop should have added at least one chunk
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        const headerChunk = chunks.shift();
        headerChunk.headerMetadata = this.header;
        headerChunk.headerMetadata.orderedChunkMetadata = [{ id: snapshotlegacy_js_1.SnapshotLegacy.header }];
        const blobs = [];
        for (const [index, chunk] of chunks.entries()) {
            const id = `${snapshotlegacy_js_1.SnapshotLegacy.body}_${index}`;
            this.header.orderedChunkMetadata.push({ id });
            blobs.push([
                id,
                (0, snapshotChunks_js_1.serializeAsMaxSupportedVersion)(id, chunk, this.logger, this.mergeTree.options, serializer, bind),
            ]);
        }
        const builder = new internal_2.SummaryTreeBuilder();
        builder.addBlob(snapshotlegacy_js_1.SnapshotLegacy.header, (0, snapshotChunks_js_1.serializeAsMaxSupportedVersion)(snapshotlegacy_js_1.SnapshotLegacy.header, headerChunk, this.logger, this.mergeTree.options, serializer, bind));
        for (const value of blobs) {
            builder.addBlob(value[0], value[1]);
        }
        return builder.getSummaryTree();
    }
    extractSync() {
        const mergeTree = this.mergeTree;
        const minSeq = this.header.minSequenceNumber;
        const minSeqStamp = { seq: minSeq, clientId: constants_js_1.NonCollabClient };
        let originalSegments = 0;
        let segmentsAfterCombine = 0;
        // Helper to add the given `MergeTreeChunkV0SegmentSpec` to the snapshot.
        const pushSegRaw = (json, length, attribution) => {
            segmentsAfterCombine += 1;
            this.segments.push(json);
            this.segmentLengths.push(length);
            if (attribution) {
                this.attributionCollections.push(attribution);
            }
        };
        // Helper to serialize the given `segment` and add it to the snapshot (if a segment is provided).
        const pushSeg = (segment) => {
            if (segment) {
                if (segment.properties !== undefined && Object.keys(segment.properties).length === 0) {
                    segment.properties = undefined;
                }
                pushSegRaw(segment.toJSONObject(), segment.cachedLength, segment.attribution);
            }
        };
        let prev;
        const extractSegment = (segment) => {
            (0, segmentInfos_js_1.assertInserted)(segment);
            // Elide segments that do not need to be included in the snapshot.  A segment may be elided if
            // either condition is true:
            //   a) The segment has not yet been ACKed.  We do not need to snapshot unACKed segments because
            //      there is a pending insert op that will deliver the segment on reconnection.
            //   b) The segment was removed at or below the MSN.  Pending ops can no longer reference this
            //      segment, and therefore we can discard it.
            if (opstampUtils.isLocal(segment.insert) ||
                ((0, segmentInfos_js_1.isRemoved)(segment) && opstampUtils.lte(segment.removes[0], minSeqStamp))) {
                if (opstampUtils.isAcked(segment.insert)) {
                    originalSegments += 1;
                }
                return true;
            }
            originalSegments += 1;
            // Next determine if the snapshot needs to preserve information required for merging the segment
            // (seq, client, etc.)  This information is only needed if the segment is above the MSN (and doesn't
            // have a pending remove.)
            if (opstampUtils.lte(segment.insert, minSeqStamp) && // Segment is below the MSN, and...
                (!(0, segmentInfos_js_1.isRemoved)(segment) || opstampUtils.isLocal(segment.removes[0]))) {
                // This segment is below the MSN, which means that future ops will not reference it.  Attempt to
                // coalesce the new segment with the previous (if any).
                if (!prev) {
                    // We do not have a previous candidate for coalescing.  Make the current segment the new candidate.
                    prev = segment;
                }
                else if (prev.canAppend(segment) &&
                    (0, properties_js_1.matchProperties)(prev.properties, segment.properties)) {
                    // We have a compatible pair.  Replace `prev` with the coalesced segment.  Clone to avoid
                    // modifying the segment instances currently in the MergeTree.
                    prev = prev.clone();
                    prev.append(segment.clone());
                }
                else {
                    // The segment pair could not be coalesced.  Record the `prev` segment in the snapshot
                    // and make the current segment the new candidate for coalescing.
                    pushSeg(prev);
                    prev = segment;
                }
            }
            else {
                // This segment needs to preserve its metadata as it may be referenced by future ops.  It's ineligible
                // for coalescing, so emit the 'prev' segment now (if any).
                pushSeg(prev);
                prev = undefined;
                if (segment.properties !== undefined && Object.keys(segment.properties).length === 0) {
                    segment.properties = undefined;
                }
                const raw = {
                    json: segment.toJSONObject(),
                };
                // If the segment insertion is above the MSN, record the insertion merge info.
                if (opstampUtils.greaterThan(segment.insert, minSeqStamp)) {
                    raw.seq = segment.insert.seq;
                    raw.client = this.getLongClientId(segment.insert.clientId);
                }
                // We have already dispensed with removed segments below the MSN and removed segments with unassigned
                // sequence numbers.  Any remaining removal info should be preserved.
                if ((0, segmentInfos_js_1.isRemoved)(segment) && segment.removes.some((r) => r.type === "setRemove")) {
                    const removes = segment.removes.filter((r) => r.type === "setRemove");
                    const firstRemove = removes[0];
                    (0, internal_1.assert)(opstampUtils.isAcked(firstRemove) &&
                        opstampUtils.greaterThan(firstRemove, minSeqStamp), 0x065 /* "On removal info preservation, segment has invalid removed sequence number!" */);
                    // TODO:AB#32299 By not preserving sequence numbers other than the first move,
                    // We drop data here which will be necessary to allow perspectives of remote clients that
                    // don't include all of their ops. This should be remedied at some point.
                    raw.removedSeq = firstRemove.seq;
                    // back compat for when we split overlap and removed client
                    // This can be removed when we can safely assume no clients running Fluid packages <= 0.58 will ever
                    // load a document produced by a version beyond the removal. It is vestigial in the meantime.
                    raw.removedClient = this.getLongClientId(firstRemove.clientId);
                    raw.removedClientIds = removes.map(({ clientId }) => this.getLongClientId(clientId));
                }
                if ((0, segmentInfos_js_1.isRemoved)(segment) && segment.removes.some((r) => r.type === "sliceRemove")) {
                    // In this format, we used the term "move" to refer to a sliceRemove/obliterate.
                    const moves = segment.removes.filter((r) => r.type === "sliceRemove");
                    const firstMove = moves[0];
                    (0, internal_1.assert)(opstampUtils.isAcked(firstMove) &&
                        opstampUtils.greaterThan(firstMove, minSeqStamp), 0x873 /* On move info preservation, segment has invalid moved sequence number! */);
                    raw.movedSeq = firstMove.seq;
                    raw.movedSeqs = moves.map(({ seq }) => seq);
                    raw.movedClientIds = moves.map(({ clientId }) => this.getLongClientId(clientId));
                }
                // Sanity check that we are preserving either the seq > minSeq or a (re)moved segment's info.
                (0, internal_1.assert)((raw.seq !== undefined && raw.client !== undefined) ||
                    (raw.removedSeq !== undefined && raw.removedClientIds !== undefined) ||
                    (raw.movedSeq !== undefined &&
                        raw.movedClientIds !== undefined &&
                        raw.movedClientIds.length > 0 &&
                        raw.movedSeqs !== undefined &&
                        raw.movedSeqs.length > 0), 0x066 /* "Corrupted preservation of segment metadata!" */);
                // Record the segment with its required metadata.
                pushSegRaw(raw, segment.cachedLength, segment.attribution);
            }
            return true;
        };
        (0, mergeTreeNodeWalk_js_1.walkAllChildSegments)(mergeTree.root, extractSegment);
        // If the last segment in the walk was coalescable, push it now.
        pushSeg(prev);
        // To reduce potential spam from this telemetry, we sample only a small
        // percentage of summaries
        if (Math.abs(originalSegments - segmentsAfterCombine) > 500 && Math.random() < 0.005) {
            this.logger.sendTelemetryEvent({
                eventName: "MergeTreeV1SummarizeSegmentCount",
                originalSegments,
                segmentsAfterCombine,
                segmentsLen: this.segments.length,
            });
        }
        return this.segments;
    }
    static async loadChunk(storage, path, logger, options, serializer) {
        const blob = await storage.readBlob(path);
        const chunkAsString = (0, client_utils_1.bufferToString)(blob, "utf8");
        return SnapshotV1.processChunk(path, chunkAsString, logger, options, serializer);
    }
    static processChunk(path, chunk, logger, options, serializer) {
        const chunkObj = serializer
            ? serializer.parse(chunk)
            : JSON.parse(chunk);
        return (0, snapshotChunks_js_1.toLatestVersion)(path, chunkObj, logger, options);
    }
}
exports.SnapshotV1 = SnapshotV1;
// Split snapshot into two entries - headers (small) and body (overflow) for faster loading initial content
// Please note that this number has no direct relationship to anything other than size of raw text (characters).
// As we produce json for the blob (and then send over the wire compressed), this number
// is really hard to correlate with any actual metric that matters (like bytes over the wire).
// For test with small number of chunks it would be closer to blob size,
// for very chunky text, blob size can easily be 4x-8x of that number.
SnapshotV1.chunkSize = 10000;
//# sourceMappingURL=snapshotV1.js.map